#!/bin/sh

# This file is part of the YateAdmin Project http://yateadmin.null.ro
# A free web interface to set up a server based on the Yate telephony engine.
# Copyright (C) 2006 Null Team
#
# Yet Another Telephony Engine - a fully featured software PBX and IVR
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

readopt()
{
    read -p "$1: [$2] " tmp
    case "x$tmp" in
	xn|xno|xN|xNO|xNo)
	    ;;
	x)
	    echo "$2"
	    ;;
	*)
	    echo "$tmp"
	    ;;
    esac
}

showhelp()
{
    cat <<EOF
    $version usage:
$0
    [--config dir] [--webpage dir]
    [--psql executable]
    [--dbhost host] [--dbname name]
    [--dbuser user] [--dbpass password]
    [--quiet]
        or one of the following unique parameters:
    help version tarball tgz tbz
EOF
}

maketarball()
{
    wd=`pwd|sed 's,^.*/,,'`
    mkdir -p tarballs
    cd ..
    tar "c${1}f" "${wd}/tarballs/${2}" $tarexclude "${wd}"
}

shortver="1"
version="Yate Administrator v$shortver"
interactive="yes"
tarexclude="--exclude CVS --exclude .cvsignore --exclude .xvpics --exclude tarballs --exclude config.php"

dbhost="localhost"
dbname="yateadmin"
dbuser="postgres"
dbpass=""

configs="`yate-config --config 2>/dev/null`"

webpage="/var/www/html"
if [ -d "$webpage" ]; then
    webpage="$webpage/yateadmin"
else
    webpage=""
fi

psqlcmd="`which psql`"

if [ "$#" = "1" ]; then
    case "x$1" in
	xtarball|xtargz|xtgz)
	    maketarball z "yateadmin-$shortver.tar.gz"
	    exit
	    ;;
	xtarbz2|xtarbz|xtbz)
	    maketarball j "yateadmin-$shortver.tar.bz2"
	    exit
	    ;;
	x*.tar.gz|x*.tgz)
	    maketarball z "$1"
	    exit
	    ;;
	x*.tar.bz2|x*.tbz)
	    maketarball j "$1"
	    exit
	    ;;
	x*.tar)
	    maketarball "" "$1"
	    exit
	    ;;
	xhelp)
	    showhelp
	    exit
	    ;;
	xversion)
	    echo "YateAdmin $shortver"
	    exit
	    ;;
    esac
fi

while [ "$#" != "0" ]; do
    cmd="$1"
    shift
    case "x$cmd" in
	x--help|x-h)
	    showhelp
	    exit
	    ;;
	x--version|x-V)
	    echo "YateAdmin $shortver"
	    exit
	    ;;
	x--quiet|x-q)
	    interactive="no"
	    ;;
	x--config|x-c)
	    configs="$1"
	    shift
	    ;;
	x--webpage|x-w)
	    webpage="$1"
	    shift
	    ;;
	x--psql)
	    psqlcmd="$1"
	    shift
	    ;;
	x--dbhost)
	    dbhost="$1"
	    shift
	    ;;
	x--dbname)
	    dbname="$1"
	    shift
	    ;;
	x--dbuser)
	    dbuser="$1"
	    shift
	    ;;
	x--dbpass)
	    dbpass="$1"
	    shift
	    ;;
	*)
	    echo "Unexpected parameter: $cmd" >&2
	    showhelp >&2
	    exit
	    ;;
    esac
done

echo "Installer for $version"

if [ "x$interactive" != "xno" ]; then
    echo "At the following prompts you can enter the word 'no' to disable defaults"
    configs=`readopt "Install Yate config file in" "$configs"`
    webpage=`readopt "Install Web pages in" "$webpage"`
    dbhost=`readopt "Database host" "$dbhost"`
    if [ -n "$dbhost" ]; then
	dbname=`readopt "Database name" "$dbname"`
	dbuser=`readopt "Database user" "$dbuser"`
	dbpass=`readopt "Database password" "$dbpass"`
	psqlcmd=`readopt "PostgreSQL command" "$psqlcmd"`
    else
	dbname=""
	dbuser=""
	dbpass=""
	psqlcmd=""
    fi
fi

if [ "x$webpage$dbhost" = "x" ]; then
    echo "Nothing to do!"
    exit
fi

echo "Install options"
cat <<EOF
    Config file in '$configs'
    Web pages in   '$webpage'
    Database:
        Host     '$dbhost'
        Name     '$dbname'
        User     '$dbuser'
        Password '$dbpass'
    PosgreSQL tool '$psqlcmd'
EOF

if [ "x$interactive" != "xno" ]; then
    if [ -z `readopt "Proceed with installation?" "yes"` ]; then
	echo "Aborting..."
	exit
    fi
fi

if [ -n "$configs" ]; then
    echo "Installing Yate configuration files"
    fd="$configs/pgsqldb.conf";
    d="
[yateadmin]
host=$dbhost
database=$dbname
user=$dbuser
password=$dbpass
"
    fr="$configs/register.conf";

    if [ -e "$fd" ]; then
	if [ -z `readopt "Overwrite existing pgsqldb.conf ?" "yes"` ]; then
	    echo "Please edit file $fd like follows:"
	    echo "$d"
	    fd=""
	fi
    fi
    if [ -n "$fd" ]; then
	echo "Creating database configuration file"
	mkdir -p "$configs"
	echo "; File created by $version
$d" > "$fd"
    fi

    if [ -e "$fr" ]; then
	if [ -z `readopt "Overwrite existing register.conf ?" "yes"` ]; then
	    echo "Please copy file $fr from the install directory then edit it if you need"
	    fr=""
	fi
    fi
    if [ -n "$fr" ]; then
	echo "Creating register configuration file"
	mkdir -p "$configs"
	(echo "; File created by $version
"; cat register.conf) > "$fr"
    fi
fi

if [ -n "$webpage" ]; then
    echo "Installing Web application"
    mkdir -p "$webpage"
    # this is a convenient way to filter what we copy
    (cd web; tar cf - $tarexclude *) | tar xf - -C "$webpage/"
    if [ -n "$dbhost" ]; then
	echo "Creating configuration file"
	cat << EOF > "$webpage/config.php"
<?
/* File created by $version */

/* Link to follow by clicking the picture on the front page */
\$link  = "http://yate.null.ro/";

/* Title displayed in browser's caption bar */
\$title = "Yate Admin";

/* Settings for connecting to the PostgreSQL database */

/* Host where the database server is running - use "localhost" for local */
\$db_host = "$dbhost";
/* Name of the database (a server may have many independent databases) */
\$db_name = "$dbname";
/* Database username to use when connecting */
\$db_user = "$dbuser";
/* Password for the database access */
\$db_pass = "$dbpass";

\$conn  = pg_connect("host='\$db_host' dbname='\$db_name' user='\$db_user' password='\$db_pass'")
    or die("Could not connect to the postgresql database");
?>
EOF
    fi
fi

if [ -n "$psqlcmd" -a -n "$dbhost" ]; then
    echo "Initializing the database"
    if [ -n "$dbpass" ]; then
	echo "At the password prompt please enter: $dbpass"
    fi
    "$psqlcmd" -h "$dbhost" -U "$dbuser" -d template1 -c "CREATE DATABASE $dbname; COMMENT ON DATABASE $dbname IS '$version'"
    "$psqlcmd" -h "$dbhost" -d "$dbname" -U "$dbuser" -f yateadmin.sql
fi
